

/* Add recipient for encryption with certificate security. */
int
add_recipient(const pstring& optlist)
{
    volatile int retval = 0;

    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_add_recipient(plop, p_optlist_param);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Close the input and output documents. */
void
close_document(int doc, const pstring& optlist)
{
    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);

    PLOPCPP_TRY {
	MPLOPLIB_API PLOP_close_document(plop, doc, p_optlist_param);
    }
    PLOPCPP_CATCH;
}


/* Create a PDF output document (which may be protected) in memory or on disk file. */
int
create_document(const pstring& filename, const pstring& optlist)
{
    volatile int retval = 0;

    std::string filename_param;
    const char *p_filename_param;
    int len_filename;
    param_to_0utf16(filename, filename_param, p_filename_param, len_filename);
    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_create_document(plop, p_filename_param, len_filename, p_optlist_param);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Create a named virtual read-only file from data provided in memory. */
void
create_pvf(const pstring& filename, const void * data, size_t size, const pstring& optlist)
{
    std::string filename_param;
    const char *p_filename_param;
    int len_filename;
    param_to_0utf16(filename, filename_param, p_filename_param, len_filename);
    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);

    PLOPCPP_TRY {
	MPLOPLIB_API PLOP_create_pvf(plop, p_filename_param, len_filename, data, size, p_optlist_param);
    }
    PLOPCPP_CATCH;
}


/* Delete a named virtual file and free its data structures (but not the contents). */
int
delete_pvf(const pstring& filename)
{
    volatile int retval = 0;

    std::string filename_param;
    const char *p_filename_param;
    int len_filename;
    param_to_0utf16(filename, filename_param, p_filename_param, len_filename);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_delete_pvf(plop, p_filename_param, len_filename);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Get the name of the API function which threw the last exception or failed. */
pstring
get_apiname()
{
    const char * volatile retval = NULL;
    pstring pstring_retval;

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_get_apiname(plop);
    }
    PLOPCPP_CATCH;

    apiretval_to_pstring(retval, pstring_retval);

    return pstring_retval;
}


/* Fetch full or partial buffer contents of the output document from memory. */
const char *
get_buffer(long *outputlen)
{
    const char *volatile retval = NULL;

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_get_buffer(plop, outputlen);
    }
    PLOPCPP_CATCH;

    return (const char*)retval;
}


/* Get the descriptive text of the last thrown exception, or the reason of a failed function call. */
pstring
get_errmsg()
{
    const char * volatile retval = NULL;
    pstring pstring_retval;

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_get_errmsg(plop);
    }
    PLOPCPP_CATCH;

    apiretval_to_pstring(retval, pstring_retval);

    return pstring_retval;
}


/* Get the number of the last thrown exception, or the reason of a failed function call. */
int
get_errnum()
{
    volatile int retval = 0;

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_get_errnum(plop);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Query properties of a virtual file or the PDFlib Virtual File system (PVF). */
double
info_pvf(const pstring& filename, const pstring& keyword)
{
    volatile double retval = 0;

    std::string filename_param;
    const char *p_filename_param;
    int len_filename;
    param_to_0utf16(filename, filename_param, p_filename_param, len_filename);
    std::string keyword_param;
    const char *p_keyword_param;
    param_to_utf8(keyword, keyword_param, p_keyword_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_info_pvf(plop, p_filename_param, len_filename, p_keyword_param);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Open a PDF document (which may be protected) for processing. */
int
open_document(const pstring& filename, const pstring& optlist)
{
    volatile int retval = 0;

    std::string filename_param;
    const char *p_filename_param;
    int len_filename;
    param_to_0utf16(filename, filename_param, p_filename_param, len_filename);
    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_open_document(plop, p_filename_param, len_filename, p_optlist_param);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Get the value of a pCOS path with type number or boolean. */
double
pcos_get_number(int doc, const pstring& path)
{
    volatile double retval = 0;

    std::string path_param;
    const char *p_path_param;
    param_to_utf8(path, path_param, p_path_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_pcos_get_number(plop, doc, "%s", p_path_param);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Get the value of a pCOS path with type name, number, string, or boolean. */
const unsigned char *
pcos_get_stream(int doc, int *outputlen, const pstring& optlist, const pstring& path)
{
    const unsigned char *volatile retval = NULL;

    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);
    std::string path_param;
    const char *p_path_param;
    param_to_utf8(path, path_param, p_path_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_pcos_get_stream(plop, doc, outputlen, p_optlist_param, "%s", p_path_param);
    }
    PLOPCPP_CATCH;

    return (const unsigned char*)retval;
}


/* Get the value of a pCOS path with type name, string, or boolean. */
pstring
pcos_get_string(int doc, const pstring& path)
{
    const char * volatile retval = NULL;
    pstring pstring_retval;

    std::string path_param;
    const char *p_path_param;
    param_to_utf8(path, path_param, p_path_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_pcos_get_string(plop, doc, "%s", p_path_param);
    }
    PLOPCPP_CATCH;

    apiretval_to_pstring(retval, pstring_retval);

    return pstring_retval;
}


/* Prepare signature options. */
int
prepare_signature(const pstring& optlist)
{
    volatile int retval = 0;

    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);

    PLOPCPP_TRY {
	retval = MPLOPLIB_API PLOP_prepare_signature(plop, p_optlist_param);
    }
    PLOPCPP_CATCH;

    return retval;
}


/* Set one or more global options for PLOP. */
void
set_option(const pstring& optlist)
{
    std::string optlist_param;
    const char *p_optlist_param;
    param_to_utf8(optlist, optlist_param, p_optlist_param);

    PLOPCPP_TRY {
	MPLOPLIB_API PLOP_set_option(plop, p_optlist_param);
    }
    PLOPCPP_CATCH;
}
